package com.faraphel.tasks_valider.ui.screen.room

import android.net.wifi.p2p.WifiP2pConfig
import android.net.wifi.p2p.WifiP2pDevice
import android.net.wifi.p2p.WifiP2pDeviceList
import android.util.Log
import androidx.compose.foundation.layout.Column
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import com.faraphel.tasks_valider.connectivity.bwf.BwfManager
import com.faraphel.tasks_valider.connectivity.room.RoomClient
import com.faraphel.tasks_valider.ui.screen.tasks.TaskGroupScreen
import com.faraphel.tasks_valider.ui.widgets.connectivity.WifiP2pDeviceListWidget
import kotlinx.coroutines.runBlocking


/**
 * This screen allow the user to join a room
 */
@Composable
fun RoomClientScreen(bwfManager: BwfManager) {
    val navController = rememberNavController()

    val selectedDevice = remember { mutableStateOf<WifiP2pDevice?>(null) }
    val connected = remember { mutableStateOf(false) }

    NavHost(navController = navController, startDestination = "roomSearch") {
        composable(route = "roomSearch") {
            // let the user pick which host he wishes to connect to
            Column {
                Text(text = "Find a Server")
                // display all the devices that are owner of their group
                WifiP2pDeviceListWidget(
                    peers = bwfManager.statePeers.value,
                    filter = { device: WifiP2pDevice -> device.isGroupOwner },
                    selectedDevice,
                )
            }

            // update the list when a new device is detected
        }
        composable(route = "taskGroup") {
            TaskGroupScreen()
        }
    }

    // if a device is selected, connect to it
    if (selectedDevice.value != null) {
        // configure the connection to point to the selected device
        val config = WifiP2pConfig().apply {
            deviceAddress = selectedDevice.value!!.deviceAddress
        }

        // try to connect to the host
        bwfManager.connect(config) {
            Log.i("room", "Connection successful to the host !")

            bwfManager.requestConnectionInfo { connectionInfo ->
                val client = RoomClient(connectionInfo.groupOwnerAddress, 9876)  // TODO(Faraphel): port should be a settings
                Thread { client.start() }.start()

                connected.value = true
            }
        }
    }
}